/* Multitasking_III.c */
#include <stdlib.h>
#include <avr/io.h>
#include "d:/AVRStudio4/C/libs/lcd2x16.h"
#define F_CPU 16000000UL
#include <util/delay.h>
#include <avr/interrupt.h>

#define KEY_DDR         DDRD
#define KEY_PORT        PORTD
#define KEY_PIN         PIND
#define KEY0            0
 
#define LED_DDR         DDRC
#define LED_PORT        PORTC
#define LED0            0

volatile uint8_t flag_1ms;

uint8_t taste_lesen(void)
{
  if (KEY_PIN & (1<<KEY0))
    return 1;
  else
    return 0;
}
 
void led_blinken(uint8_t taste)
{
  static uint16_t zaehler=0;
  
  if (taste)
  {
    if (zaehler>=5000)
	{
	  LED_PORT ^= (1<<LED0);
	  zaehler=0;
    }
  }
  else
  {
    if (zaehler>=2500)
	{
	  LED_PORT ^= (1<<LED0);
	  zaehler=0;
    }
  }
  zaehler++;
}

void lcd_ausgabe(void)
{
  static uint16_t zaehler=0;
  
  if (zaehler>=10000)
  {
	zaehler=0;
	lcd_puts("*    Start    *\n");
  }
  zaehler++;
}

ISR ( TIMER2_COMP_vect )
{
  if (flag_1ms)	// Laufzeit der Tasks > 2 ms => Fehlersignalisierung
  {
    LED_PORT |= (1<<LED0);	// LED0 an, Programm stoppen
	while (1);
  }
  flag_1ms = 1;
}


int main(void)
{
  int8_t taste;
  
  // LCD initialisieren
  lcd_init(LCD_DISP_ON);	// initialisiere Display
  lcd_clrscr();

  // Taste initialisieren
  KEY_DDR &= (0<<KEY0);	// Taste 0 ist Eingang

  // LED initialisieren
  LED_DDR |= (1<<LED0);	// LED0 ist Ausgang
  LED_PORT |= (0<<LED0); // LED0 ausschalten
  
  // Timer2 initialisieren
  TCCR2 = (1<<WGM21) | (1<<CS01) | (1<<CS00);	// CTC, Prescaler 64
  OCR2  = 56;	// 1ms
  TIMSK |= (1<<OCIE2);
  
  // Interrupts global freigeben
  sei();

  while (1)
  {    
	if (flag_1ms)
	{
	  flag_1ms=0;
	  
	  // abzuarbeitende Tasks
	  taste = taste_lesen();
	  led_blinken(taste);
	  lcd_ausgabe();
	  
	  if (flag_1ms)	// Laufzeit der Tasks > 1 ms => Fehlersignalisierung
	  {
		LED_PORT |= (1<<LED0);	// LED0 an, Programm stoppen
		while(1);
      }
    }
  }
}
